%module flwt

%{
/* System header files */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>   /* exit() */
/* FLWT header files */
#include <constant.h>
#include <flwtdef.h>
#include <flwterr.h>
#include <filter.h>
#include <flwt.h>
#include <imgmem.h>
#include <lift.h>
#include <mallat.h>
#include <mem.h>
#include <util.h>
%}

typedef double Flt;

/* Static Global Variables */
static Vector filter;      /* Vector with filter coefficients */
static Matrix liftingX;    /* Matrix with lifting coefficients in X */
static Matrix liftingY;    /* Matrix with lifting coefficients in Y */

%include "../include/constant.h"
%include "../include/flwtdef.h"
%include "../include/filter.h"
%include "../include/flwt.h"
%include "../include/imgmem.h"
%include "../include/lift.h"
%include "../include/mallat.h"
%include "../include/mem.h"
%include "../include/util.h"

%inline %{
/*
 * FLWTFilter1D function: applies the given factor beta to the given set of
 *                        coefficients.
 */
static void
FLWTFilter1D ( Vector Data, const int len, const int incr, const int step,
                            const Flt beta )
{
    register int gammaIdx;
    int stepIncr, size;

    /* Initialize some variable */
    stepIncr = (step*incr) << 1;
    gammaIdx = stepIncr >> 1;
    size = len*incr;

    /* Step through Gammas and apply Beta factor */
    while (gammaIdx < size) {
        Data[gammaIdx] *= beta;
        gammaIdx += stepIncr;
    }
}

/*
 * FLWTFilter1DC function: applies a circular filter to the given set of
 *                         wavelet coefficients.
 */
static void
FLWTFilter1DC ( Vector Data, const int len, const int incr, const int step,
                             const Flt betaStart, const Flt betaStop,
			     const int xy, const int levels )
{
    register Vector gammaPtr;
    int stepIncr, x, step2, k;
    Flt i, j, s, beta, interval = betaStop - betaStart;

    if ( xy > len ) {
        Error ( "FLWTFilter1DC", INVALID_RELATIONAL_INT, ABORT, "xy", "<=", len );
    }

    /* Initialize some variables */
    x = 0;
    step2 = step << 1;
    stepIncr = step2*incr;
    gammaPtr = Data + (stepIncr >> 1);

    /* Step through Gammas and apply Beta factor */
    for ( x=0 ; x<len ; x+=step2 ) {
        Flt dist;

	i = (Flt)x / (Flt)len - (Flt)0.5;
	j = (Flt)xy / (Flt)len - (Flt)0.5;
        dist = j*j + i*i;
	s = dist*(Flt)len/(Flt)255;   /* s lies b/w 0 and 1 */
	if ( betaStart <= betaStop ) {
	    s = s*interval + betaStart; 
        } else {
            s = (Flt)1 - s;
            s = betaStop - s*interval;
        }
	beta = s;
	for ( k=0 ; k<levels ; k++ )
	    beta *= s;
        *(gammaPtr) *= beta;
        gammaPtr += stepIncr;
    }
}

%}
